import json
import logging
from functools import wraps
from typing import Any, Dict, Optional, Type

from pydantic import BaseModel
from requests import Response, Session

DEFAULT_API_TIMEOUT = 30


def validate_schema(schema: Type[BaseModel]):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            req_data = kwargs.pop("data", None) or args[1]
            if isinstance(req_data, dict):
                obj = schema.parse_obj(req_data)
                req_data = json.loads(obj.json(exclude_none=True))
            elif isinstance(req_data, schema):
                req_data = json.loads(req_data.json(exclude_none=True))
            else:
                raise ValueError(
                    f"'data' argument must be dict or instance of model {schema}"
                )
            return func(args[0], req_data, **kwargs)

        return wrapper

    return decorator


class WrongRequestException(Exception):
    pass


class HttpClient:
    base_url = None

    def _fetch_response(
        self,
        path: str,
        method: str,
        params: Optional[Dict] = None,
        data: Optional[Any] = None,
        json: Optional[Dict] = None,
        headers: Optional[Dict] = None,
        timeout: Optional[int] = None,
    ):
        url = self.base_url + path

        request_data = {
            "params": params,
            "data": data,
            "json": json,
            "allow_redirects": False,
            "headers": headers or {},
            "timeout": timeout or DEFAULT_API_TIMEOUT,
        }
        try:
            session = Session()
            if method == "GET":
                response = session.get(url, **request_data)
            elif method == "POST":
                response = session.post(url, **request_data)
            elif method == "PATCH":
                response = session.patch(url, **request_data)
            elif method == "PUT":
                response = session.put(url, **request_data)
            else:
                raise WrongRequestException(f"Unsupported method type {method}")
        except Exception as e:
            logging.error("An error occurred while sending the request")
            raise
        return response

    def get(
        self,
        path: str,
        params: Optional[Dict] = None,
        headers: Optional[Dict] = None,
        timeout: Optional[int] = None,
    ) -> Response:
        return self._fetch_response(
            path, method="GET", params=params, headers=headers, timeout=timeout
        )

    def post(
        self,
        path: str,
        data: Optional[Any] = None,
        headers: Optional[Dict] = None,
        timeout: Optional[int] = None,
    ) -> Response:
        return self._fetch_response(
            path, method="POST", json=data, headers=headers, timeout=timeout
        )

    def patch(
        self,
        path: str,
        data: Optional[Any] = None,
        headers: Optional[Dict] = None,
        timeout: Optional[int] = None,
    ) -> Response:
        return self._fetch_response(
            path, method="PATCH", json=data, headers=headers, timeout=timeout
        )

    def put(
        self,
        path: str,
        data: Optional[Any] = None,
        headers: Optional[Dict] = None,
        timeout: Optional[int] = None,
    ) -> Response:
        return self._fetch_response(
            path, method="PUT", json=data, headers=headers, timeout=timeout
        )



# generated by datamodel-codegen:
#   filename:  entities.yaml
#   timestamp: 2023-02-13T15:29:46+00:00

from __future__ import annotations

from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Optional

from pydantic import AnyUrl, BaseModel, Field


class Tag(BaseModel):
    name: str


class BooleanFieldStat(BaseModel):
    true_count: int
    false_count: int
    nulls_count: int


class ComplexFieldStat(BaseModel):
    nulls_count: int
    unique_count: int


class NumberFieldStat(BaseModel):
    low_value: float
    high_value: float
    mean_value: Optional[float] = None
    median_value: Optional[float] = None
    nulls_count: int
    unique_count: int


class StringFieldStat(BaseModel):
    max_length: int
    avg_length: float
    nulls_count: int
    unique_count: int


class BinaryFieldStat(BaseModel):
    max_length: int
    avg_length: float
    nulls_count: int
    unique_count: int


class IntegerFieldStat(BaseModel):
    low_value: int
    high_value: int
    mean_value: Optional[int] = None
    median_value: Optional[int] = None
    nulls_count: int
    unique_count: int


class DateTimeFieldStat(BaseModel):
    low_value: datetime
    high_value: datetime
    mean_value: Optional[datetime] = None
    median_value: Optional[datetime] = None
    nulls_count: int
    unique_count: int


class Type(Enum):
    TYPE_STRING = 'TYPE_STRING'
    TYPE_NUMBER = 'TYPE_NUMBER'
    TYPE_INTEGER = 'TYPE_INTEGER'
    TYPE_BOOLEAN = 'TYPE_BOOLEAN'
    TYPE_CHAR = 'TYPE_CHAR'
    TYPE_DATETIME = 'TYPE_DATETIME'
    TYPE_TIME = 'TYPE_TIME'
    TYPE_STRUCT = 'TYPE_STRUCT'
    TYPE_BINARY = 'TYPE_BINARY'
    TYPE_LIST = 'TYPE_LIST'
    TYPE_MAP = 'TYPE_MAP'
    TYPE_UNION = 'TYPE_UNION'
    TYPE_DURATION = 'TYPE_DURATION'
    TYPE_UNKNOWN = 'TYPE_UNKNOWN'


class DataSetFieldType(BaseModel):
    type: Type
    logical_type: Optional[str] = None
    is_nullable: bool


class DataTransformer(BaseModel):
    source_code_url: Optional[str] = None
    sql: Optional[str] = None
    inputs: List[str]
    outputs: List[str]


class DataConsumer(BaseModel):
    inputs: Optional[List[str]] = None


class DataInput(BaseModel):
    outputs: Optional[List[str]] = None


class DataEntityGroup(BaseModel):
    entities_list: List[str]
    group_oddrn: Optional[str] = None


class DataQualityTestExpectation(BaseModel):
    type: Optional[str] = Field(None, example='expect_table_row_count_to_be_between')


class LinkedUrl(BaseModel):
    name: str
    url: str


class JobRunStatus(Enum):
    SUCCESS = 'SUCCESS'
    FAILED = 'FAILED'
    SKIPPED = 'SKIPPED'
    BROKEN = 'BROKEN'
    ABORTED = 'ABORTED'
    RUNNING = 'RUNNING'
    UNKNOWN = 'UNKNOWN'


class QualityRunStatus(Enum):
    SUCCESS = 'SUCCESS'
    FAILED = 'FAILED'
    SKIPPED = 'SKIPPED'
    BROKEN = 'BROKEN'
    ABORTED = 'ABORTED'
    RUNNING = 'RUNNING'
    UNKNOWN = 'UNKNOWN'


class DataTransformerRun(BaseModel):
    transformer_oddrn: str
    start_time: datetime
    end_time: Optional[datetime] = None
    status_reason: Optional[str] = None
    status: JobRunStatus


class DataQualityTestRun(BaseModel):
    data_quality_test_oddrn: str
    start_time: datetime
    end_time: Optional[datetime] = None
    status_reason: Optional[str] = None
    status: QualityRunStatus


class DataEntityType(Enum):
    TABLE = 'TABLE'
    VIEW = 'VIEW'
    FILE = 'FILE'
    FEATURE_GROUP = 'FEATURE_GROUP'
    KAFKA_TOPIC = 'KAFKA_TOPIC'
    JOB = 'JOB'
    JOB_RUN = 'JOB_RUN'
    ML_MODEL = 'ML_MODEL'
    ML_MODEL_TRAINING = 'ML_MODEL_TRAINING'
    DASHBOARD = 'DASHBOARD'
    DAG = 'DAG'
    ML_EXPERIMENT = 'ML_EXPERIMENT'
    GRAPH_NODE = 'GRAPH_NODE'
    UNKNOWN = 'UNKNOWN'
    MICROSERVICE = 'MICROSERVICE'
    API_CALL = 'API_CALL'
    DATABASE_SERVICE = 'DATABASE_SERVICE'
    API_SERVICE = 'API_SERVICE'
    KAFKA_SERVICE = 'KAFKA_SERVICE'


class MetadataExtension(BaseModel):
    schema_url: AnyUrl = Field(
        ...,
        description='The JSON Pointer (https://tools.ietf.org/html/rfc6901) URL to the corresponding version of the schema definition for this extension',
        example='https://raw.githubusercontent.com/opendatadiscovery/opendatadiscovery-specification/main/specification/extensions/glue.json#/definitions/GlueDataSetExtension',
    )
    metadata: Dict[str, Any]


class DataSource(BaseModel):
    oddrn: str
    name: str
    description: Optional[str] = None


class DataSourceList(BaseModel):
    items: List[DataSource]


class IngestionAlertType(Enum):
    DATA_SOURCE_CONNECTION_ERROR = 'DATA_SOURCE_CONNECTION_ERROR'
    DATA_SOURCE_GENERIC_ERROR = 'DATA_SOURCE_GENERIC_ERROR'


class IngestionAlert(BaseModel):
    description: str
    type: IngestionAlertType
    related_data_source_oddrn: str
    related_data_entity_oddrn: Optional[str] = None


class IngestionAlertList(BaseModel):
    items: List[IngestionAlert]


class CompactDataEntity(BaseModel):
    oddrn: str
    type: DataEntityType


class CompactDataEntityList(BaseModel):
    items: List[CompactDataEntity]


class BaseObject(BaseModel):
    oddrn: str = Field(..., example='//aws/glue/{account_id}/{database}/{tablename}')
    name: str
    version: Optional[str] = None
    description: Optional[str] = None
    owner: Optional[str] = Field(None, example='//aws/iam/{account_id}/user/name')
    metadata: Optional[List[MetadataExtension]] = None
    tags: Optional[List[Tag]] = None


class DataSetFieldStat(BaseModel):
    tags: Optional[List[Tag]] = None
    complex_stats: Optional[ComplexFieldStat] = None
    boolean_stats: Optional[BooleanFieldStat] = None
    integer_stats: Optional[IntegerFieldStat] = None
    number_stats: Optional[NumberFieldStat] = None
    string_stats: Optional[StringFieldStat] = None
    binary_stats: Optional[BinaryFieldStat] = None
    datetime_stats: Optional[DateTimeFieldStat] = None


class DataSetField(BaseObject):
    parent_field_oddrn: Optional[str] = None
    type: DataSetFieldType
    is_primary_key: Optional[bool] = None
    is_sort_key: Optional[bool] = None
    is_key: Optional[bool] = None
    is_value: Optional[bool] = None
    default_value: Optional[str] = None
    stats: Optional[DataSetFieldStat] = None


class DataQualityTest(BaseModel):
    suite_name: str
    dataset_list: List[str]
    expectation: DataQualityTestExpectation
    suite_url: Optional[str] = None
    linked_url_list: Optional[List[LinkedUrl]] = None


class DataSet(BaseModel):
    parent_oddrn: Optional[str] = None
    rows_number: Optional[int] = None
    field_list: List[DataSetField]


class DataSetStatistics(BaseModel):
    dataset_oddrn: str
    fields: Dict[str, DataSetFieldStat] = Field(
        ...,
        description='key is a field oddrn. For example: "//aws/glue/{account_id}/{database}/{tablename}/columns/{column_name}"\n',
    )


class DataEntity(BaseObject):
    updated_at: Optional[datetime] = None
    created_at: Optional[datetime] = None
    type: DataEntityType
    dataset: Optional[DataSet] = None
    data_transformer: Optional[DataTransformer] = None
    data_transformer_run: Optional[DataTransformerRun] = None
    data_quality_test: Optional[DataQualityTest] = None
    data_quality_test_run: Optional[DataQualityTestRun] = None
    data_input: Optional[DataInput] = None
    data_consumer: Optional[DataConsumer] = None
    data_entity_group: Optional[DataEntityGroup] = None


class DataEntityList(BaseModel):
    data_source_oddrn: str = Field(..., example='//aws/glue/123456789010/')
    items: Optional[List[DataEntity]] = None


class DatasetStatisticsList(BaseModel):
    items: List[DataSetStatistics]



from typing import List, Optional

import sql_metadata
from sql_metadata import Parser, QueryType

DDL = {QueryType.CREATE, QueryType.DROP, QueryType.ALTER, QueryType.SELECT}
DML = {QueryType.INSERT, QueryType.UPDATE, QueryType.DELETE}


class ParseSqlError(Exception):
    ...


class UnsupportedQueryType(ParseSqlError):
    ...


class SqlStatement:
    def __init__(self, sql: str, skip_wc: bool = True):
        self.inputs: Optional[List[str]] = []
        self.outputs: Optional[List[str]] = []
        self.comments: Optional[List[str]] = []
        self.skip_wc = skip_wc

        self._columns: Optional[List[str]] = []
        self._sql = sql

        self.parse_sql()

    @property
    def columns(self) -> List[str]:
        return self._columns

    @columns.setter
    def columns(self, columns: List[str]):
        if self.skip_wc:
            self._columns = [col for col in columns if "*" not in col]
        else:
            self._columns = columns

    def parse_sql(self) -> None:
        parser = sql_metadata.Parser(self._sql)

        try:
            query_type: str = parser.query_type

            if query_type in DDL:
                self.__parse_ddl(parser)
            elif query_type in DML:
                self.__parse_dml(parser)
            else:
                raise ValueError(f"Unsupported type {query_type}")

            self.comments = parser.comments

        except ValueError as e:
            raise UnsupportedQueryType from e

    def __parse_dml(self, parser: Parser):
        self.outputs = parser.tables
        self.columns = parser.columns

    def __parse_ddl(self, parser: Parser):
        self.inputs = parser.tables
        self.columns = parser.columns



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['odd_models', 'odd_models.api_client']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.9.1,<2.0.0',
 'requests==2.27.1',
 'sql-metadata>=2.6.0,<3.0.0',
 'sqlparse==0.4.2']

setup_kwargs = {
    'name': 'odd-models',
    'version': '2.0.12',
    'description': 'Open Data Discovery Models',
    'long_description': "# Open Data Discovery models package\n\nModels automatically generated by `datamodel-code-generator` from [ODD OpenApi specification](https://github.com/opendatadiscovery/opendatadiscovery-specification)\n\n\n## Usage example:\n```python\nfrom odd_models.models import DataEntityList\n\ndata_entity_list = DataEntityList(data_source_oddrn='/postgresql/host/localhost/databases/opendatadiscovery', items=[])\n```",
    'author': 'Open Data Discovery',
    'author_email': 'pypi@opendatadiscovery.org',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/opendatadiscovery/odd-models-package',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from .api_client.open_data_discovery_ingestion_api import ODDApiClient
from requests import Response
from .models import (
    DataEntityList,
    DataSet,
    MetadataExtension,
    DataSourceList,
    DataSource,
)
from typing import List, Dict, Any


class OddIntegrator:
    source_name = "internal_host"
    service_name = "internal_service"

    def __init__(self, odd_platform_url: str, odd_collector_token: str):
        self.odd_collector_token = odd_collector_token
        self.platform_client = ODDApiClient(base_url=odd_platform_url)

    @property
    def source_oddrn(self) -> str:
        return f"//{self.service_name}/{self.source_name}"

    def register_source(self) -> int:
        reg_source_data = DataSourceList(
            items=[
                DataSource(
                    oddrn=self.source_oddrn, name=self.source_name, description=None
                )
            ]
        )
        reg_headers = {
            "content-type": "application/json",
            "Authorization": f"Bearer {self.odd_collector_token}",
        }

        resp: Response = self.platform_client.create_data_source(
            data=reg_source_data, headers=reg_headers
        )
        return resp.status_code

    @staticmethod
    def create_metadata_extension_list(
        metadata: Dict[str, Any],
        schema_url: str = "https://raw.githubusercontent.com/opendatadiscovery/",
    ) -> List[MetadataExtension]:
        return [
            MetadataExtension(
                schema_url=schema_url,
                metadata={key: value for key, value in metadata.items()},
            )
        ]

    def send_data_entity(self, dataset: DataSet) -> int:
        odd_request = DataEntityList(
            items=[dataset], data_source_oddrn=self.source_oddrn
        )
        resp = self.platform_client.post_data_entity_list(odd_request)
        return resp



from typing import Optional, Dict, Union

from odd_models import models
from odd_models.api_client.http_client import HttpClient, validate_schema
from pydantic import BaseModel


class ODDApiClient(HttpClient):
    base_url = None

    def __init__(self, base_url: str) -> None:
        self.base_url = base_url.rstrip('/')

    @validate_schema(models.IngestionAlertList)
    def create_alerts(
            self,
            data: Union[dict, BaseModel],
            headers: Optional[Dict] = None,
            timeout: Optional[int] = None,
    ):
        request_data = {
            'path': '/ingestion/alerts',
            'data': data,
            'headers': headers,
            'timeout': timeout,
        }
        return self.post(**request_data)

    @validate_schema(models.DataSourceList)
    def create_data_source(
            self,
            data: Union[dict, BaseModel],
            headers: Optional[Dict] = None,
            timeout: Optional[int] = None,
    ):
        request_data = {
            'path': '/ingestion/datasources',
            'data': data,
            'headers': headers,
            'timeout': timeout,
        }
        return self.post(**request_data)

    def get_data_entities_by_deg_oddrn(
            self,
            oddrn,
            headers: Optional[Dict] = None,
            timeout: Optional[int] = None,
    ):
        request_data = {
            'path': '/ingestion/entities/degs/children',
            'params': {
                'oddrn': oddrn,
            },
            'headers': headers,
            'timeout': timeout,
        }
        return self.get(**request_data)

    @validate_schema(models.DataEntityList)
    def post_data_entity_list(
            self,
            data: Union[dict, BaseModel],
            headers: Optional[Dict] = None,
            timeout: Optional[int] = None,
    ):
        request_data = {
            'path': '/ingestion/entities',
            'data': data,
            'headers': headers,
            'timeout': timeout,
        }
        return self.post(**request_data)

    @validate_schema(models.DatasetStatisticsList)
    def post_data_set_stats_list(
            self,
            data: Union[dict, BaseModel],
            headers: Optional[Dict] = None,
            timeout: Optional[int] = None,
    ):
        request_data = {
            'path': '/ingestion/entities/datasets/stats',
            'data': data,
            'headers': headers,
            'timeout': timeout,
        }
        return self.post(**request_data)

